
import numpy as np
import xarray

import gasunits

import ddeq.background
import ddeq.misc


def compute_plume_signal(data, trace_gas):
    """
    Compute plume signal for trace gas
    """
    name = f'{trace_gas}_minus_estimated_background'
    signal = data[trace_gas] - data[f'{trace_gas}_estimated_background']

    data[name] = xarray.DataArray(signal, dims=data[trace_gas].dims,
                                  attrs=data[trace_gas].attrs)
    return data


def prepare_data(data, trace_gas='CO2'):
    """
    Prepare mass-balance approach:
    - estimate CO2 and NO2 background
    - convert units to mass column densities (kg/m²)
    """
    data[f'{trace_gas}_isfinite'] = np.isfinite(data[trace_gas])

    # estimate background
    data = ddeq.background.estimate(data, trace_gas)
    

    # compute CO2/NO2 enhancement
    data = compute_plume_signal(data, trace_gas)
    
    # convert ppm to kg/m2
    for variable in [trace_gas, f'{trace_gas}_estimated_background', f'{trace_gas}_minus_estimated_background']:
    
        values = data[variable]
        attrs = values.attrs

        name = f'{variable}_mass'
        
        if data[trace_gas].attrs['units'] == 'molecules cm-2':
            input_unit = 'cm-2'
        elif data[trace_gas].attrs['units'] == 'ppm':
            input_unit = 'ppmv'
        else:
            raise ValueError('No units in dataset')
            
        
        data[name] = xarray.DataArray(
            gasunits.convert_columns(values, input_unit, 'kg m-2',
                                     p=data['psurf'], molar_mass=trace_gas),
            dims=values.dims, attrs=values.attrs
        )
            
        if 'noise_level' in attrs and not isinstance(attrs['noise_level'], str):
            attrs['noise_level'] = gasunits.convert_columns(attrs['noise_level'],
                                                            input_unit, 'kg m-2',
                                                            molar_mass=trace_gas,
                                                            p=np.nanmean(data['psurf'])),
        data[name].attrs.update(attrs)
        data[name].attrs['units'] = 'kg m-2'    
            
    return data
